@@ -12,6 +12,8 @@ if MYPY:
     from typing import Optional
     from typing import Tuple
     from typing import Type
+    from typing import Union
+    from typing_extensions import Literal
 
     ExcInfo = Tuple[
         Optional[Type[BaseException]], Optional[BaseException], Optional[TracebackType]
@@ -23,9 +25,26 @@ if MYPY:
     Breadcrumb = Dict[str, Any]
     BreadcrumbHint = Dict[str, Any]
 
+    SamplingContext = Dict[str, Any]
+
     EventProcessor = Callable[[Event, Hint], Optional[Event]]
     ErrorProcessor = Callable[[Event, ExcInfo], Optional[Event]]
     BreadcrumbProcessor = Callable[[Breadcrumb, BreadcrumbHint], Optional[Breadcrumb]]
 
+    TracesSampler = Callable[[SamplingContext], Union[float, int, bool]]
+
     # https://github.com/python/mypy/issues/5710
     NotImplementedType = Any
+
+    EventDataCategory = Literal[
+        "default",
+        "error",
+        "crash",
+        "transaction",
+        "security",
+        "attachment",
+        "session",
+        "internal",
+    ]
+    SessionStatus = Literal["ok", "exited", "crashed", "abnormal"]
+    EndpointType = Literal["store", "envelope"]
