@@ -1,6 +1,8 @@
 from sentry_sdk._types import MYPY
 
 if MYPY:
+    import sentry_sdk
+
     from typing import Optional
     from typing import Callable
     from typing import Union
@@ -9,11 +11,34 @@ if MYPY:
     from typing import Dict
     from typing import Any
     from typing import Sequence
+    from typing_extensions import TypedDict
 
-    from sentry_sdk.transport import Transport
     from sentry_sdk.integrations import Integration
 
-    from sentry_sdk._types import Event, EventProcessor, BreadcrumbProcessor
+    from sentry_sdk._types import (
+        BreadcrumbProcessor,
+        Event,
+        EventProcessor,
+        TracesSampler,
+    )
+
+    # Experiments are feature flags to enable and disable certain unstable SDK
+    # functionality. Changing them from the defaults (`None`) in production
+    # code is highly discouraged. They are not subject to any stability
+    # guarantees such as the ones from semantic versioning.
+    Experiments = TypedDict(
+        "Experiments",
+        {
+            "max_spans": Optional[int],
+            "record_sql_params": Optional[bool],
+            "smart_transaction_trimming": Optional[bool],
+            "propagate_tracestate": Optional[bool],
+        },
+        total=False,
+    )
+
+DEFAULT_QUEUE_SIZE = 100
+DEFAULT_MAX_BREADCRUMBS = 100
 
 
 # This type exists to trick mypy and PyCharm into thinking `init` and `Client`
@@ -23,7 +48,7 @@ class ClientConstructor(object):
         self,
         dsn=None,  # type: Optional[str]
         with_locals=True,  # type: bool
-        max_breadcrumbs=100,  # type: int
+        max_breadcrumbs=DEFAULT_MAX_BREADCRUMBS,  # type: int
         release=None,  # type: Optional[str]
         environment=None,  # type: Optional[str]
         server_name=None,  # type: Optional[str]
@@ -33,7 +58,8 @@ class ClientConstructor(object):
         in_app_exclude=[],  # type: List[str]  # noqa: B006
         default_integrations=True,  # type: bool
         dist=None,  # type: Optional[str]
-        transport=None,  # type: Optional[Union[Transport, Type[Transport], Callable[[Event], None]]]
+        transport=None,  # type: Optional[Union[sentry_sdk.transport.Transport, Type[sentry_sdk.transport.Transport], Callable[[Event], None]]]
+        transport_queue_size=DEFAULT_QUEUE_SIZE,  # type: int
         sample_rate=1.0,  # type: float
         send_default_pii=False,  # type: bool
         http_proxy=None,  # type: Optional[str]
@@ -46,10 +72,12 @@ class ClientConstructor(object):
         attach_stacktrace=False,  # type: bool
         ca_certs=None,  # type: Optional[str]
         propagate_traces=True,  # type: bool
-        # DO NOT ENABLE THIS RIGHT NOW UNLESS YOU WANT TO EXCEED YOUR EVENT QUOTA IMMEDIATELY
-        traces_sample_rate=0.0,  # type: float
-        traceparent_v2=False,  # type: bool
-        _experiments={},  # type: Dict[str, Any]  # noqa: B006
+        traces_sample_rate=None,  # type: Optional[float]
+        traces_sampler=None,  # type: Optional[TracesSampler]
+        auto_enabling_integrations=True,  # type: bool
+        auto_session_tracking=True,  # type: bool
+        send_client_reports=True,  # type: bool
+        _experiments={},  # type: Experiments  # noqa: B006
     ):
         # type: (...) -> None
         pass
@@ -65,14 +93,15 @@ def _get_default_options():
         getargspec = inspect.getargspec  # type: ignore
 
     a = getargspec(ClientConstructor.__init__)
-    return dict(zip(a.args[-len(a.defaults) :], a.defaults))
+    defaults = a.defaults or ()
+    return dict(zip(a.args[-len(defaults) :], defaults))
 
 
 DEFAULT_OPTIONS = _get_default_options()
 del _get_default_options
 
 
-VERSION = "0.13.2"
+VERSION = "1.4.3"
 SDK_INFO = {
     "name": "sentry.python",
     "version": VERSION,
