@@ -1,5 +1,4 @@
 import copy
-import random
 import sys
 
 from datetime import datetime
@@ -8,7 +7,8 @@ from contextlib import contextmanager
 from sentry_sdk._compat import with_metaclass
 from sentry_sdk.scope import Scope
 from sentry_sdk.client import Client
-from sentry_sdk.tracing import Span
+from sentry_sdk.tracing import Span, Transaction
+from sentry_sdk.session import Session
 from sentry_sdk.utils import (
     exc_info_from_error,
     event_from_exception,
@@ -23,6 +23,7 @@ if MYPY:
     from typing import Any
     from typing import Optional
     from typing import Tuple
+    from typing import Dict
     from typing import List
     from typing import Callable
     from typing import Generator
@@ -32,7 +33,13 @@ if MYPY:
     from typing import ContextManager
 
     from sentry_sdk.integrations import Integration
-    from sentry_sdk._types import Event, Hint, Breadcrumb, BreadcrumbHint, ExcInfo
+    from sentry_sdk._types import (
+        Event,
+        Hint,
+        Breadcrumb,
+        BreadcrumbHint,
+        ExcInfo,
+    )
     from sentry_sdk.consts import ClientConstructor
 
     T = TypeVar("T")
@@ -47,6 +54,24 @@ else:
 _local = ContextVar("sentry_current_hub")
 
 
+def _update_scope(base, scope_change, scope_kwargs):
+    # type: (Scope, Optional[Any], Dict[str, Any]) -> Scope
+    if scope_change and scope_kwargs:
+        raise TypeError("cannot provide scope and kwargs")
+    if scope_change is not None:
+        final_scope = copy.copy(base)
+        if callable(scope_change):
+            scope_change(final_scope)
+        else:
+            final_scope.update_from_scope(scope_change)
+    elif scope_kwargs:
+        final_scope = copy.copy(base)
+        final_scope.update_from_kwargs(**scope_kwargs)
+    else:
+        final_scope = base
+    return final_scope
+
+
 def _should_send_default_pii():
     # type: () -> bool
     client = Hub.current.client
@@ -92,7 +117,7 @@ if MYPY:
     # Use `ClientConstructor` to define the argument types of `init` and
     # `ContextManager[Any]` to tell static analyzers about the return type.
 
-    class init(ClientConstructor, ContextManager[Any]):
+    class init(ClientConstructor, ContextManager[Any]):  # noqa: N801
         pass
 
 
@@ -250,7 +275,7 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         else:
             raise ValueError("Integration has no name")
 
-        client = self._stack[-1][0]
+        client = self.client
         if client is not None:
             rv = client.integrations.get(integration_name)
             if rv is not None:
@@ -285,14 +310,17 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         self,
         event,  # type: Event
         hint=None,  # type: Optional[Hint]
+        scope=None,  # type: Optional[Any]
+        **scope_args  # type: Any
     ):
         # type: (...) -> Optional[str]
-        """Captures an event. Alias of :py:meth:`sentry_sdk.Client.capture_event`.
-        """
-        client, scope = self._stack[-1]
+        """Captures an event. Alias of :py:meth:`sentry_sdk.Client.capture_event`."""
+        client, top_scope = self._stack[-1]
+        scope = _update_scope(top_scope, scope, scope_args)
         if client is not None:
+            is_transaction = event.get("type") == "transaction"
             rv = client.capture_event(event, hint, scope)
-            if rv is not None:
+            if rv is not None and not is_transaction:
                 self._last_event_id = rv
             return rv
         return None
@@ -301,6 +329,8 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         self,
         message,  # type: str
         level=None,  # type: Optional[str]
+        scope=None,  # type: Optional[Any]
+        **scope_args  # type: Any
     ):
         # type: (...) -> Optional[str]
         """Captures a message.  The message is just a string.  If no level
@@ -312,10 +342,15 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
             return None
         if level is None:
             level = "info"
-        return self.capture_event({"message": message, "level": level})
+        return self.capture_event(
+            {"message": message, "level": level}, scope=scope, **scope_args
+        )
 
     def capture_exception(
-        self, error=None  # type: Optional[Union[BaseException, ExcInfo]]
+        self,
+        error=None,  # type: Optional[Union[BaseException, ExcInfo]]
+        scope=None,  # type: Optional[Any]
+        **scope_args  # type: Any
     ):
         # type: (...) -> Optional[str]
         """Captures an exception.
@@ -334,7 +369,7 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
         event, hint = event_from_exception(exc_info, client_options=client.options)
         try:
-            return self.capture_event(event, hint=hint)
+            return self.capture_event(event, hint=hint, scope=scope, **scope_args)
         except Exception:
             self._capture_internal_exception(sys.exc_info())
 
@@ -405,48 +440,106 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
     ):
         # type: (...) -> Span
         """
-        Create a new span whose parent span is the currently active
-        span, if any. The return value is the span object that can
-        be used as a context manager to start and stop timing.
+        Create and start timing a new span whose parent is the currently active
+        span or transaction, if any. The return value is a span instance,
+        typically used as a context manager to start and stop timing in a `with`
+        block.
 
-        Note that you will not see any span that is not contained
-        within a transaction. Create a transaction with
-        ``start_span(transaction="my transaction")`` if an
-        integration doesn't already do this for you.
+        Only spans contained in a transaction are sent to Sentry. Most
+        integrations start a transaction at the appropriate time, for example
+        for every incoming HTTP request. Use `start_transaction` to start a new
+        transaction when one is not already in progress.
         """
+        # TODO: consider removing this in a future release.
+        # This is for backwards compatibility with releases before
+        # start_transaction existed, to allow for a smoother transition.
+        if isinstance(span, Transaction) or "transaction" in kwargs:
+            deprecation_msg = (
+                "Deprecated: use start_transaction to start transactions and "
+                "Transaction.start_child to start spans."
+            )
+            if isinstance(span, Transaction):
+                logger.warning(deprecation_msg)
+                return self.start_transaction(span)
+            if "transaction" in kwargs:
+                logger.warning(deprecation_msg)
+                name = kwargs.pop("transaction")
+                return self.start_transaction(name=name, **kwargs)
 
-        client, scope = self._stack[-1]
+        if span is not None:
+            return span
 
         kwargs.setdefault("hub", self)
 
-        if span is None:
-            span = scope.span
-            if span is not None:
-                span = span.new_span(**kwargs)
-            else:
-                span = Span(**kwargs)
+        span = self.scope.span
+        if span is not None:
+            return span.start_child(**kwargs)
 
-        if span.sampled is None and span.transaction is not None:
-            sample_rate = client and client.options["traces_sample_rate"] or 0
-            span.sampled = random.random() < sample_rate
+        return Span(**kwargs)
 
-        if span.sampled:
+    def start_transaction(
+        self,
+        transaction=None,  # type: Optional[Transaction]
+        **kwargs  # type: Any
+    ):
+        # type: (...) -> Transaction
+        """
+        Start and return a transaction.
+
+        Start an existing transaction if given, otherwise create and start a new
+        transaction with kwargs.
+
+        This is the entry point to manual tracing instrumentation.
+
+        A tree structure can be built by adding child spans to the transaction,
+        and child spans to other spans. To start a new child span within the
+        transaction or any span, call the respective `.start_child()` method.
+
+        Every child span must be finished before the transaction is finished,
+        otherwise the unfinished spans are discarded.
+
+        When used as context managers, spans and transactions are automatically
+        finished at the end of the `with` block. If not using context managers,
+        call the `.finish()` method.
+
+        When the transaction is finished, it will be sent to Sentry with all its
+        finished child spans.
+        """
+        custom_sampling_context = kwargs.pop("custom_sampling_context", {})
+
+        # if we haven't been given a transaction, make one
+        if transaction is None:
+            kwargs.setdefault("hub", self)
+            transaction = Transaction(**kwargs)
+
+        # use traces_sample_rate, traces_sampler, and/or inheritance to make a
+        # sampling decision
+        sampling_context = {
+            "transaction_context": transaction.to_json(),
+            "parent_sampled": transaction.parent_sampled,
+        }
+        sampling_context.update(custom_sampling_context)
+        transaction._set_initial_sampling_decision(sampling_context=sampling_context)
+
+        # we don't bother to keep spans if we already know we're not going to
+        # send the transaction
+        if transaction.sampled:
             max_spans = (
-                client and client.options["_experiments"].get("max_spans") or 1000
-            )
-            span.init_finished_spans(maxlen=max_spans)
+                self.client and self.client.options["_experiments"].get("max_spans")
+            ) or 1000
+            transaction.init_span_recorder(maxlen=max_spans)
 
-        return span
+        return transaction
 
-    @overload  # noqa
-    def push_scope(
+    @overload
+    def push_scope(  # noqa: F811
         self, callback=None  # type: Optional[None]
     ):
         # type: (...) -> ContextManager[Scope]
         pass
 
-    @overload  # noqa
-    def push_scope(
+    @overload
+    def push_scope(  # noqa: F811
         self, callback  # type: Callable[[Scope], None]
     ):
         # type: (...) -> None
@@ -465,7 +558,6 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         :returns: If no `callback` is provided, a context manager that should
             be used to pop the scope again.
         """
-
         if callback is not None:
             with self.push_scope() as scope:
                 callback(scope)
@@ -488,15 +580,15 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         assert self._stack, "stack must have at least one layer"
         return rv
 
-    @overload  # noqa
-    def configure_scope(
+    @overload
+    def configure_scope(  # noqa: F811
         self, callback=None  # type: Optional[None]
     ):
         # type: (...) -> ContextManager[Scope]
         pass
 
-    @overload  # noqa
-    def configure_scope(
+    @overload
+    def configure_scope(  # noqa: F811
         self, callback  # type: Callable[[Scope], None]
     ):
         # type: (...) -> None
@@ -532,6 +624,52 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
 
         return inner()
 
+    def start_session(
+        self, session_mode="application"  # type: str
+    ):
+        # type: (...) -> None
+        """Starts a new session."""
+        self.end_session()
+        client, scope = self._stack[-1]
+        scope._session = Session(
+            release=client.options["release"] if client else None,
+            environment=client.options["environment"] if client else None,
+            user=scope._user,
+            session_mode=session_mode,
+        )
+
+    def end_session(self):
+        # type: (...) -> None
+        """Ends the current session if there is one."""
+        client, scope = self._stack[-1]
+        session = scope._session
+        self.scope._session = None
+
+        if session is not None:
+            session.close()
+            if client is not None:
+                client.capture_session(session)
+
+    def stop_auto_session_tracking(self):
+        # type: (...) -> None
+        """Stops automatic session tracking.
+
+        This temporarily session tracking for the current scope when called.
+        To resume session tracking call `resume_auto_session_tracking`.
+        """
+        self.end_session()
+        client, scope = self._stack[-1]
+        scope._force_auto_session_tracking = False
+
+    def resume_auto_session_tracking(self):
+        # type: (...) -> None
+        """Resumes automatic session tracking for the current scope if
+        disabled earlier.  This requires that generally automatic session
+        tracking is enabled.
+        """
+        client, scope = self._stack[-1]
+        scope._force_auto_session_tracking = None
+
     def flush(
         self,
         timeout=None,  # type: Optional[float]
@@ -545,25 +683,25 @@ class Hub(with_metaclass(HubMeta)):  # type: ignore
         if client is not None:
             return client.flush(timeout=timeout, callback=callback)
 
-    def iter_trace_propagation_headers(self):
-        # type: () -> Generator[Tuple[str, str], None, None]
-        # TODO: Document
-        client, scope = self._stack[-1]
-        span = scope.span
-
-        if span is None:
+    def iter_trace_propagation_headers(self, span=None):
+        # type: (Optional[Span]) -> Generator[Tuple[str, str], None, None]
+        """
+        Return HTTP headers which allow propagation of trace data. Data taken
+        from the span representing the request, if available, or the current
+        span on the scope if not.
+        """
+        span = span or self.scope.span
+        if not span:
             return
 
+        client = self._stack[-1][0]
+
         propagate_traces = client and client.options["propagate_traces"]
         if not propagate_traces:
             return
 
-        if client and client.options["traceparent_v2"]:
-            traceparent = span.to_traceparent()
-        else:
-            traceparent = span.to_legacy_traceparent()
-
-        yield "sentry-trace", traceparent
+        for header in span.iter_headers():
+            yield header
 
 
 GLOBAL_HUB = Hub()
