@@ -9,53 +9,85 @@ from sentry_sdk.utils import logger
 from sentry_sdk._types import MYPY
 
 if MYPY:
-    from typing import Iterator
+    from typing import Callable
     from typing import Dict
+    from typing import Iterator
     from typing import List
     from typing import Set
+    from typing import Tuple
     from typing import Type
-    from typing import Callable
 
 
 _installer_lock = Lock()
 _installed_integrations = set()  # type: Set[str]
 
 
-def _generate_default_integrations_iterator(*import_strings):
-    # type: (*str) -> Callable[[], Iterator[Type[Integration]]]
-    def iter_default_integrations():
-        # type: () -> Iterator[Type[Integration]]
-        """Returns an iterator of the default integration classes:
-        """
+def _generate_default_integrations_iterator(integrations, auto_enabling_integrations):
+    # type: (Tuple[str, ...], Tuple[str, ...]) -> Callable[[bool], Iterator[Type[Integration]]]
+
+    def iter_default_integrations(with_auto_enabling_integrations):
+        # type: (bool) -> Iterator[Type[Integration]]
+        """Returns an iterator of the default integration classes:"""
         from importlib import import_module
 
-        for import_string in import_strings:
-            module, cls = import_string.rsplit(".", 1)
-            yield getattr(import_module(module), cls)
+        if with_auto_enabling_integrations:
+            all_import_strings = integrations + auto_enabling_integrations
+        else:
+            all_import_strings = integrations
+
+        for import_string in all_import_strings:
+            try:
+                module, cls = import_string.rsplit(".", 1)
+                yield getattr(import_module(module), cls)
+            except (DidNotEnable, SyntaxError) as e:
+                logger.debug(
+                    "Did not import default integration %s: %s", import_string, e
+                )
 
     if isinstance(iter_default_integrations.__doc__, str):
-        for import_string in import_strings:
+        for import_string in integrations:
             iter_default_integrations.__doc__ += "\n- `{}`".format(import_string)
 
     return iter_default_integrations
 
 
+_AUTO_ENABLING_INTEGRATIONS = (
+    "sentry_sdk.integrations.django.DjangoIntegration",
+    "sentry_sdk.integrations.flask.FlaskIntegration",
+    "sentry_sdk.integrations.bottle.BottleIntegration",
+    "sentry_sdk.integrations.falcon.FalconIntegration",
+    "sentry_sdk.integrations.sanic.SanicIntegration",
+    "sentry_sdk.integrations.celery.CeleryIntegration",
+    "sentry_sdk.integrations.rq.RqIntegration",
+    "sentry_sdk.integrations.aiohttp.AioHttpIntegration",
+    "sentry_sdk.integrations.tornado.TornadoIntegration",
+    "sentry_sdk.integrations.sqlalchemy.SqlalchemyIntegration",
+    "sentry_sdk.integrations.boto3.Boto3Integration",
+)
+
+
 iter_default_integrations = _generate_default_integrations_iterator(
-    "sentry_sdk.integrations.logging.LoggingIntegration",
-    "sentry_sdk.integrations.stdlib.StdlibIntegration",
-    "sentry_sdk.integrations.excepthook.ExcepthookIntegration",
-    "sentry_sdk.integrations.dedupe.DedupeIntegration",
-    "sentry_sdk.integrations.atexit.AtexitIntegration",
-    "sentry_sdk.integrations.modules.ModulesIntegration",
-    "sentry_sdk.integrations.argv.ArgvIntegration",
-    "sentry_sdk.integrations.threading.ThreadingIntegration",
+    integrations=(
+        # stdlib/base runtime integrations
+        "sentry_sdk.integrations.logging.LoggingIntegration",
+        "sentry_sdk.integrations.stdlib.StdlibIntegration",
+        "sentry_sdk.integrations.excepthook.ExcepthookIntegration",
+        "sentry_sdk.integrations.dedupe.DedupeIntegration",
+        "sentry_sdk.integrations.atexit.AtexitIntegration",
+        "sentry_sdk.integrations.modules.ModulesIntegration",
+        "sentry_sdk.integrations.argv.ArgvIntegration",
+        "sentry_sdk.integrations.threading.ThreadingIntegration",
+    ),
+    auto_enabling_integrations=_AUTO_ENABLING_INTEGRATIONS,
 )
 
 del _generate_default_integrations_iterator
 
 
-def setup_integrations(integrations, with_defaults=True):
-    # type: (List[Integration], bool) -> Dict[str, Integration]
+def setup_integrations(
+    integrations, with_defaults=True, with_auto_enabling_integrations=False
+):
+    # type: (List[Integration], bool, bool) -> Dict[str, Integration]
     """Given a list of integration instances this installs them all.  When
     `with_defaults` is set to `True` then all default integrations are added
     unless they were already provided before.
@@ -66,11 +98,17 @@ def setup_integrations(integrations, with_defaults=True):
 
     logger.debug("Setting up integrations (with default = %s)", with_defaults)
 
+    # Integrations that are not explicitly set up by the user.
+    used_as_default_integration = set()
+
     if with_defaults:
-        for integration_cls in iter_default_integrations():
+        for integration_cls in iter_default_integrations(
+            with_auto_enabling_integrations
+        ):
             if integration_cls.identifier not in integrations:
                 instance = integration_cls()
                 integrations[instance.identifier] = instance
+                used_as_default_integration.add(instance.identifier)
 
     for identifier, integration in iteritems(integrations):
         with _installer_lock:
@@ -90,6 +128,14 @@ def setup_integrations(integrations, with_defaults=True):
                         integration.install()
                     else:
                         raise
+                except DidNotEnable as e:
+                    if identifier not in used_as_default_integration:
+                        raise
+
+                    logger.debug(
+                        "Did not enable default integration %s: %s", identifier, e
+                    )
+
                 _installed_integrations.add(identifier)
 
     for identifier in integrations:
@@ -98,6 +144,16 @@ def setup_integrations(integrations, with_defaults=True):
     return integrations
 
 
+class DidNotEnable(Exception):
+    """
+    The integration could not be enabled due to a trivial user error like
+    `flask` not being installed for the `FlaskIntegration`.
+
+    This exception is silently swallowed for default integrations, but reraised
+    for explicitly enabled integrations.
+    """
+
+
 class Integration(object):
     """Baseclass for all integrations.
 
