@@ -43,6 +43,9 @@ else:
     from pyramid.security import authenticated_userid  # type: ignore
 
 
+TRANSACTION_STYLE_VALUES = ("route_name", "route_pattern")
+
+
 class PyramidIntegration(Integration):
     identifier = "pyramid"
 
@@ -50,7 +53,6 @@ class PyramidIntegration(Integration):
 
     def __init__(self, transaction_style="route_name"):
         # type: (str) -> None
-        TRANSACTION_STYLE_VALUES = ("route_name", "route_pattern")
         if transaction_style not in TRANSACTION_STYLE_VALUES:
             raise ValueError(
                 "Invalid value for transaction_style: %s (must be in %s)"
@@ -61,24 +63,33 @@ class PyramidIntegration(Integration):
     @staticmethod
     def setup_once():
         # type: () -> None
-        from pyramid.router import Router
+        from pyramid import router
         from pyramid.request import Request
 
-        old_handle_request = Router.handle_request
+        old_call_view = router._call_view
 
-        def sentry_patched_handle_request(self, request, *args, **kwargs):
+        def sentry_patched_call_view(registry, request, *args, **kwargs):
             # type: (Any, Request, *Any, **Any) -> Response
             hub = Hub.current
             integration = hub.get_integration(PyramidIntegration)
+
             if integration is not None:
                 with hub.configure_scope() as scope:
+                    try:
+                        if integration.transaction_style == "route_name":
+                            scope.transaction = request.matched_route.name
+                        elif integration.transaction_style == "route_pattern":
+                            scope.transaction = request.matched_route.pattern
+                    except Exception:
+                        pass
+
                     scope.add_event_processor(
                         _make_event_processor(weakref.ref(request), integration)
                     )
 
-            return old_handle_request(self, request, *args, **kwargs)
+            return old_call_view(registry, request, *args, **kwargs)
 
-        Router.handle_request = sentry_patched_handle_request
+        router._call_view = sentry_patched_call_view
 
         if hasattr(Request, "invoke_exception_view"):
             old_invoke_exception_view = Request.invoke_exception_view
@@ -99,7 +110,7 @@ class PyramidIntegration(Integration):
 
             Request.invoke_exception_view = sentry_patched_invoke_exception_view
 
-        old_wsgi_call = Router.__call__
+        old_wsgi_call = router.Router.__call__
 
         def sentry_patched_wsgi_call(self, environ, start_response):
             # type: (Any, Dict[str, str], Callable[..., Any]) -> _ScopedResponse
@@ -121,7 +132,7 @@ class PyramidIntegration(Integration):
                 environ, start_response
             )
 
-        Router.__call__ = sentry_patched_wsgi_call
+        router.Router.__call__ = sentry_patched_wsgi_call
 
 
 def _capture_exception(exc_info):
@@ -194,21 +205,13 @@ def _make_event_processor(weak_request, integration):
         if request is None:
             return event
 
-        try:
-            if integration.transaction_style == "route_name":
-                event["transaction"] = request.matched_route.name
-            elif integration.transaction_style == "route_pattern":
-                event["transaction"] = request.matched_route.pattern
-        except Exception:
-            pass
-
         with capture_internal_exceptions():
             PyramidRequestExtractor(request).extract_into_event(event)
 
         if _should_send_default_pii():
             with capture_internal_exceptions():
                 user_info = event.setdefault("user", {})
-                user_info["id"] = authenticated_userid(request)
+                user_info.setdefault("id", authenticated_userid(request))
 
         return event
 
