@@ -3,26 +3,29 @@ from __future__ import absolute_import
 import weakref
 
 from sentry_sdk.hub import Hub
-from sentry_sdk.integrations import Integration
-from sentry_sdk.tracing import Span
+from sentry_sdk.integrations import DidNotEnable, Integration
+from sentry_sdk.integrations.logging import ignore_logger
+from sentry_sdk.tracing import Transaction
 from sentry_sdk.utils import capture_internal_exceptions, event_from_exception
 
-from rq.timeouts import JobTimeoutException
-from rq.worker import Worker
-from rq.queue import Queue
+try:
+    from rq.queue import Queue
+    from rq.timeouts import JobTimeoutException
+    from rq.version import VERSION as RQ_VERSION
+    from rq.worker import Worker
+except ImportError:
+    raise DidNotEnable("RQ not installed")
 
 from sentry_sdk._types import MYPY
 
 if MYPY:
-    from typing import Any
-    from typing import Dict
-    from typing import Callable
+    from typing import Any, Callable, Dict
+
+    from sentry_sdk._types import EventProcessor
+    from sentry_sdk.utils import ExcInfo
 
     from rq.job import Job
 
-    from sentry_sdk.utils import ExcInfo
-    from sentry_sdk._types import EventProcessor
-
 
 class RqIntegration(Integration):
     identifier = "rq"
@@ -31,6 +34,14 @@ class RqIntegration(Integration):
     def setup_once():
         # type: () -> None
 
+        try:
+            version = tuple(map(int, RQ_VERSION.split(".")[:3]))
+        except (ValueError, TypeError):
+            raise DidNotEnable("Unparsable RQ version: {}".format(RQ_VERSION))
+
+        if version < (0, 6):
+            raise DidNotEnable("RQ 0.6 or newer is required.")
+
         old_perform_job = Worker.perform_job
 
         def sentry_patched_perform_job(self, job, *args, **kwargs):
@@ -48,15 +59,18 @@ class RqIntegration(Integration):
                 scope.clear_breadcrumbs()
                 scope.add_event_processor(_make_event_processor(weakref.ref(job)))
 
-                span = Span.continue_from_headers(
-                    job.meta.get("_sentry_trace_headers") or {}
+                transaction = Transaction.continue_from_headers(
+                    job.meta.get("_sentry_trace_headers") or {},
+                    op="rq.task",
+                    name="unknown RQ task",
                 )
-                span.op = "rq.task"
 
                 with capture_internal_exceptions():
-                    span.transaction = job.func_name
+                    transaction.name = job.func_name
 
-                with hub.start_span(span):
+                with hub.start_transaction(
+                    transaction, custom_sampling_context={"rq_job": job}
+                ):
                     rv = old_perform_job(self, job, *args, **kwargs)
 
             if self.is_horse:
@@ -73,7 +87,9 @@ class RqIntegration(Integration):
 
         def sentry_patched_handle_exception(self, job, *exc_info, **kwargs):
             # type: (Worker, Any, *Any, **Any) -> Any
-            _capture_exception(exc_info)  # type: ignore
+            if job.is_failed:
+                _capture_exception(exc_info)  # type: ignore
+
             return old_handle_exception(self, job, *exc_info, **kwargs)
 
         Worker.handle_exception = sentry_patched_handle_exception
@@ -92,6 +108,8 @@ class RqIntegration(Integration):
 
         Queue.enqueue_job = sentry_patched_enqueue_job
 
+        ignore_logger("rq.worker")
+
 
 def _make_event_processor(weak_job):
     # type: (Callable[[], Job]) -> EventProcessor
