@@ -6,8 +6,8 @@ import platform
 from sentry_sdk.hub import Hub
 from sentry_sdk.integrations import Integration
 from sentry_sdk.scope import add_global_event_processor
-from sentry_sdk.tracing import EnvironHeaders, record_http_request
-from sentry_sdk.utils import capture_internal_exceptions, safe_repr
+from sentry_sdk.tracing_utils import EnvironHeaders
+from sentry_sdk.utils import capture_internal_exceptions, logger, safe_repr
 
 from sentry_sdk._types import MYPY
 
@@ -78,48 +78,38 @@ def _install_httplib():
                 url,
             )
 
-        recorder = record_http_request(hub, real_url, method)
-        data_dict = recorder.__enter__()
+        span = hub.start_span(op="http", description="%s %s" % (method, real_url))
 
-        try:
-            rv = real_putrequest(self, method, url, *args, **kwargs)
+        span.set_data("method", method)
+        span.set_data("url", real_url)
 
-            for key, value in hub.iter_trace_propagation_headers():
-                self.putheader(key, value)
-        except Exception:
-            recorder.__exit__(*sys.exc_info())
-            raise
+        rv = real_putrequest(self, method, url, *args, **kwargs)
 
-        self._sentrysdk_recorder = recorder
-        self._sentrysdk_data_dict = data_dict
+        for key, value in hub.iter_trace_propagation_headers(span):
+            logger.debug(
+                "[Tracing] Adding `{key}` header {value} to outgoing request to {real_url}.".format(
+                    key=key, value=value, real_url=real_url
+                )
+            )
+            self.putheader(key, value)
+
+        self._sentrysdk_span = span
 
         return rv
 
     def getresponse(self, *args, **kwargs):
         # type: (HTTPConnection, *Any, **Any) -> Any
-        recorder = getattr(self, "_sentrysdk_recorder", None)
+        span = getattr(self, "_sentrysdk_span", None)
 
-        if recorder is None:
+        if span is None:
             return real_getresponse(self, *args, **kwargs)
 
-        data_dict = getattr(self, "_sentrysdk_data_dict", None)
+        rv = real_getresponse(self, *args, **kwargs)
 
-        try:
-            rv = real_getresponse(self, *args, **kwargs)
-
-            if data_dict is not None:
-                data_dict["status_code"] = rv.status
-                data_dict["reason"] = rv.reason
-        except TypeError:
-            # python-requests provokes a typeerror to discover py3 vs py2 differences
-            #
-            # > TypeError("getresponse() got an unexpected keyword argument 'buffering'")
-            raise
-        except Exception:
-            recorder.__exit__(*sys.exc_info())
-            raise
-        else:
-            recorder.__exit__(None, None, None)
+        span.set_data("status_code", rv.status)
+        span.set_http_status(int(rv.status))
+        span.set_data("reason", rv.reason)
+        span.finish()
 
         return rv
 
@@ -193,12 +183,15 @@ def _install_subprocess():
 
         env = None
 
-        for k, v in hub.iter_trace_propagation_headers():
-            if env is None:
-                env = _init_argument(a, kw, "env", 10, lambda x: dict(x or os.environ))
-            env["SUBPROCESS_" + k.upper().replace("-", "_")] = v
-
         with hub.start_span(op="subprocess", description=description) as span:
+
+            for k, v in hub.iter_trace_propagation_headers(span):
+                if env is None:
+                    env = _init_argument(
+                        a, kw, "env", 10, lambda x: dict(x or os.environ)
+                    )
+                env["SUBPROCESS_" + k.upper().replace("-", "_")] = v
+
             if cwd:
                 span.set_data("subprocess.cwd", cwd)
 
