@@ -1,9 +1,19 @@
+import asyncio
 import json
+from contextlib import suppress
 
+import pytest
 from aiohttp import web
+from aiohttp.client import ServerDisconnectedError
+from aiohttp.web_request import Request
 
 from sentry_sdk.integrations.aiohttp import AioHttpIntegration
 
+try:
+    from unittest import mock  # python 3.3 and above
+except ImportError:
+    import mock  # python < 3.3
+
 
 async def test_basic(sentry_init, aiohttp_client, loop, capture_events):
     sentry_init(integrations=[AioHttpIntegration()])
@@ -20,14 +30,14 @@ async def test_basic(sentry_init, aiohttp_client, loop, capture_events):
     resp = await client.get("/")
     assert resp.status == 500
 
-    event, = events
+    (event,) = events
 
     assert (
         event["transaction"]
         == "tests.integrations.aiohttp.test_aiohttp.test_basic.<locals>.hello"
     )
 
-    exception, = event["exception"]["values"]
+    (exception,) = event["exception"]["values"]
     assert exception["type"] == "ZeroDivisionError"
     request = event["request"]
     host = request["headers"]["Host"]
@@ -64,8 +74,8 @@ async def test_post_body_not_read(sentry_init, aiohttp_client, loop, capture_eve
     resp = await client.post("/", json=body)
     assert resp.status == 500
 
-    event, = events
-    exception, = event["exception"]["values"]
+    (event,) = events
+    (exception,) = event["exception"]["values"]
     assert exception["type"] == "ZeroDivisionError"
     request = event["request"]
 
@@ -92,8 +102,8 @@ async def test_post_body_read(sentry_init, aiohttp_client, loop, capture_events)
     resp = await client.post("/", json=body)
     assert resp.status == 500
 
-    event, = events
-    exception, = event["exception"]["values"]
+    (event,) = events
+    (exception,) = event["exception"]["values"]
     assert exception["type"] == "ZeroDivisionError"
     request = event["request"]
 
@@ -120,6 +130,28 @@ async def test_403_not_captured(sentry_init, aiohttp_client, loop, capture_event
     assert not events
 
 
+async def test_cancelled_error_not_captured(
+    sentry_init, aiohttp_client, loop, capture_events
+):
+    sentry_init(integrations=[AioHttpIntegration()])
+
+    async def hello(request):
+        raise asyncio.CancelledError()
+
+    app = web.Application()
+    app.router.add_get("/", hello)
+
+    events = capture_events()
+    client = await aiohttp_client(app)
+
+    with suppress(ServerDisconnectedError):
+        # Intended `aiohttp` interaction: server will disconnect if it
+        # encounters `asyncio.CancelledError`
+        await client.get("/")
+
+    assert not events
+
+
 async def test_half_initialized(sentry_init, aiohttp_client, loop, capture_events):
     sentry_init(integrations=[AioHttpIntegration()])
     sentry_init()
@@ -154,10 +186,78 @@ async def test_tracing(sentry_init, aiohttp_client, loop, capture_events):
     resp = await client.get("/")
     assert resp.status == 200
 
-    event, = events
+    (event,) = events
 
     assert event["type"] == "transaction"
     assert (
         event["transaction"]
         == "tests.integrations.aiohttp.test_aiohttp.test_tracing.<locals>.hello"
     )
+
+
+@pytest.mark.parametrize(
+    "transaction_style,expected_transaction",
+    [
+        (
+            "handler_name",
+            "tests.integrations.aiohttp.test_aiohttp.test_transaction_style.<locals>.hello",
+        ),
+        ("method_and_path_pattern", "GET /{var}"),
+    ],
+)
+async def test_transaction_style(
+    sentry_init, aiohttp_client, capture_events, transaction_style, expected_transaction
+):
+    sentry_init(
+        integrations=[AioHttpIntegration(transaction_style=transaction_style)],
+        traces_sample_rate=1.0,
+    )
+
+    async def hello(request):
+        return web.Response(text="hello")
+
+    app = web.Application()
+    app.router.add_get(r"/{var}", hello)
+
+    events = capture_events()
+
+    client = await aiohttp_client(app)
+    resp = await client.get("/1")
+    assert resp.status == 200
+
+    (event,) = events
+
+    assert event["type"] == "transaction"
+    assert event["transaction"] == expected_transaction
+
+
+async def test_traces_sampler_gets_request_object_in_sampling_context(
+    sentry_init,
+    aiohttp_client,
+    DictionaryContaining,  # noqa:N803
+    ObjectDescribedBy,  # noqa:N803
+):
+    traces_sampler = mock.Mock()
+    sentry_init(
+        integrations=[AioHttpIntegration()],
+        traces_sampler=traces_sampler,
+    )
+
+    async def kangaroo_handler(request):
+        return web.Response(text="dogs are great")
+
+    app = web.Application()
+    app.router.add_get("/tricks/kangaroo", kangaroo_handler)
+
+    client = await aiohttp_client(app)
+    await client.get("/tricks/kangaroo")
+
+    traces_sampler.assert_any_call(
+        DictionaryContaining(
+            {
+                "aiohttp_request": ObjectDescribedBy(
+                    type=Request, attrs={"method": "GET", "path": "/tricks/kangaroo"}
+                )
+            }
+        )
+    )
