@@ -1,5 +1,6 @@
 import json
 import pytest
+import logging
 
 from io import BytesIO
 
@@ -11,10 +12,12 @@ from flask.views import View
 from flask_login import LoginManager, login_user
 
 from sentry_sdk import (
+    set_tag,
     configure_scope,
     capture_message,
     capture_exception,
     last_event_id,
+    Hub,
 )
 from sentry_sdk.integrations.logging import LoggingIntegration
 import sentry_sdk.integrations.flask as flask_sentry
@@ -39,6 +42,16 @@ def app():
     return app
 
 
+@pytest.fixture(params=("auto", "manual"))
+def integration_enabled_params(request):
+    if request.param == "auto":
+        return {"auto_enabling_integrations": True}
+    elif request.param == "manual":
+        return {"integrations": [flask_sentry.FlaskIntegration()]}
+    else:
+        raise ValueError(request.param)
+
+
 def test_has_context(sentry_init, app, capture_events):
     sentry_init(integrations=[flask_sentry.FlaskIntegration()])
     events = capture_events()
@@ -47,7 +60,7 @@ def test_has_context(sentry_init, app, capture_events):
     response = client.get("/message")
     assert response.status_code == 200
 
-    event, = events
+    (event,) = events
     assert event["transaction"] == "hi"
     assert "data" not in event["request"]
     assert event["request"]["url"] == "http://localhost/message"
@@ -70,14 +83,22 @@ def test_transaction_style(
     response = client.get("/message")
     assert response.status_code == 200
 
-    event, = events
+    (event,) = events
     assert event["transaction"] == expected_transaction
 
 
 @pytest.mark.parametrize("debug", (True, False))
 @pytest.mark.parametrize("testing", (True, False))
-def test_errors(sentry_init, capture_exceptions, capture_events, app, debug, testing):
-    sentry_init(integrations=[flask_sentry.FlaskIntegration()], debug=True)
+def test_errors(
+    sentry_init,
+    capture_exceptions,
+    capture_events,
+    app,
+    debug,
+    testing,
+    integration_enabled_params,
+):
+    sentry_init(debug=True, **integration_enabled_params)
 
     app.debug = debug
     app.testing = testing
@@ -95,15 +116,17 @@ def test_errors(sentry_init, capture_exceptions, capture_events, app, debug, tes
     except ZeroDivisionError:
         pass
 
-    exc, = exceptions
+    (exc,) = exceptions
     assert isinstance(exc, ZeroDivisionError)
 
-    event, = events
+    (event,) = events
     assert event["exception"]["values"][0]["mechanism"]["type"] == "flask"
 
 
-def test_flask_login_not_installed(sentry_init, app, capture_events, monkeypatch):
-    sentry_init(integrations=[flask_sentry.FlaskIntegration()])
+def test_flask_login_not_installed(
+    sentry_init, app, capture_events, monkeypatch, integration_enabled_params
+):
+    sentry_init(**integration_enabled_params)
 
     monkeypatch.setattr(flask_sentry, "flask_login", None)
 
@@ -112,12 +135,14 @@ def test_flask_login_not_installed(sentry_init, app, capture_events, monkeypatch
     client = app.test_client()
     client.get("/message")
 
-    event, = events
+    (event,) = events
     assert event.get("user", {}).get("id") is None
 
 
-def test_flask_login_not_configured(sentry_init, app, capture_events, monkeypatch):
-    sentry_init(integrations=[flask_sentry.FlaskIntegration()])
+def test_flask_login_not_configured(
+    sentry_init, app, capture_events, monkeypatch, integration_enabled_params
+):
+    sentry_init(**integration_enabled_params)
 
     assert flask_sentry.flask_login
 
@@ -125,14 +150,14 @@ def test_flask_login_not_configured(sentry_init, app, capture_events, monkeypatc
     client = app.test_client()
     client.get("/message")
 
-    event, = events
+    (event,) = events
     assert event.get("user", {}).get("id") is None
 
 
 def test_flask_login_partially_configured(
-    sentry_init, app, capture_events, monkeypatch
+    sentry_init, app, capture_events, monkeypatch, integration_enabled_params
 ):
-    sentry_init(integrations=[flask_sentry.FlaskIntegration()])
+    sentry_init(**integration_enabled_params)
 
     events = capture_events()
 
@@ -142,19 +167,22 @@ def test_flask_login_partially_configured(
     client = app.test_client()
     client.get("/message")
 
-    event, = events
+    (event,) = events
     assert event.get("user", {}).get("id") is None
 
 
 @pytest.mark.parametrize("send_default_pii", [True, False])
 @pytest.mark.parametrize("user_id", [None, "42", 3])
 def test_flask_login_configured(
-    send_default_pii, sentry_init, app, user_id, capture_events, monkeypatch
+    send_default_pii,
+    sentry_init,
+    app,
+    user_id,
+    capture_events,
+    monkeypatch,
+    integration_enabled_params,
 ):
-    sentry_init(
-        send_default_pii=send_default_pii,
-        integrations=[flask_sentry.FlaskIntegration()],
-    )
+    sentry_init(send_default_pii=send_default_pii, **integration_enabled_params)
 
     class User(object):
         is_authenticated = is_active = True
@@ -182,7 +210,7 @@ def test_flask_login_configured(
 
     assert client.get("/message").status_code == 200
 
-    event, = events
+    (event,) = events
     if user_id is None or not send_default_pii:
         assert event.get("user", {}).get("id") is None
     else:
@@ -208,13 +236,54 @@ def test_flask_large_json_request(sentry_init, capture_events, app):
     response = client.post("/", content_type="application/json", data=json.dumps(data))
     assert response.status_code == 200
 
-    event, = events
+    (event,) = events
     assert event["_meta"]["request"]["data"]["foo"]["bar"] == {
         "": {"len": 2000, "rem": [["!limit", "x", 509, 512]]}
     }
     assert len(event["request"]["data"]["foo"]["bar"]) == 512
 
 
+def test_flask_session_tracking(sentry_init, capture_envelopes, app):
+    sentry_init(
+        integrations=[flask_sentry.FlaskIntegration()],
+        release="demo-release",
+    )
+
+    @app.route("/")
+    def index():
+        with configure_scope() as scope:
+            scope.set_user({"ip_address": "1.2.3.4", "id": "42"})
+        try:
+            raise ValueError("stuff")
+        except Exception:
+            logging.exception("stuff happened")
+        1 / 0
+
+    envelopes = capture_envelopes()
+
+    with app.test_client() as client:
+        try:
+            client.get("/", headers={"User-Agent": "blafasel/1.0"})
+        except ZeroDivisionError:
+            pass
+
+    Hub.current.client.flush()
+
+    (first_event, error_event, session) = envelopes
+    first_event = first_event.get_event()
+    error_event = error_event.get_event()
+    session = session.items[0].payload.json
+    aggregates = session["aggregates"]
+
+    assert first_event["exception"]["values"][0]["type"] == "ValueError"
+    assert error_event["exception"]["values"][0]["type"] == "ZeroDivisionError"
+
+    assert len(aggregates) == 1
+    assert aggregates[0]["crashed"] == 1
+    assert aggregates[0]["started"]
+    assert session["attrs"]["release"] == "demo-release"
+
+
 @pytest.mark.parametrize("data", [{}, []], ids=["empty-dict", "empty-list"])
 def test_flask_empty_json_request(sentry_init, capture_events, app, data):
     sentry_init(integrations=[flask_sentry.FlaskIntegration()])
@@ -233,7 +302,7 @@ def test_flask_empty_json_request(sentry_init, capture_events, app, data):
     response = client.post("/", content_type="application/json", data=json.dumps(data))
     assert response.status_code == 200
 
-    event, = events
+    (event,) = events
     assert event["request"]["data"] == data
 
 
@@ -256,13 +325,46 @@ def test_flask_medium_formdata_request(sentry_init, capture_events, app):
     response = client.post("/", data=data)
     assert response.status_code == 200
 
-    event, = events
+    (event,) = events
     assert event["_meta"]["request"]["data"]["foo"] == {
         "": {"len": 2000, "rem": [["!limit", "x", 509, 512]]}
     }
     assert len(event["request"]["data"]["foo"]) == 512
 
 
+def test_flask_formdata_request_appear_transaction_body(
+    sentry_init, capture_events, app
+):
+    """
+    Test that ensures that transaction request data contains body, even if no exception was raised
+    """
+    sentry_init(integrations=[flask_sentry.FlaskIntegration()], traces_sample_rate=1.0)
+
+    data = {"username": "sentry-user", "age": "26"}
+
+    @app.route("/", methods=["POST"])
+    def index():
+        assert request.form["username"] == data["username"]
+        assert request.form["age"] == data["age"]
+        assert not request.get_data()
+        assert not request.get_json()
+        set_tag("view", "yes")
+        capture_message("hi")
+        return "ok"
+
+    events = capture_events()
+
+    client = app.test_client()
+    response = client.post("/", data=data)
+    assert response.status_code == 200
+
+    event, transaction_event = events
+
+    assert "request" in transaction_event
+    assert "data" in transaction_event["request"]
+    assert transaction_event["request"]["data"] == data
+
+
 @pytest.mark.parametrize("input_char", [u"a", b"a"])
 def test_flask_too_large_raw_request(sentry_init, input_char, capture_events, app):
     sentry_init(integrations=[flask_sentry.FlaskIntegration()], request_bodies="small")
@@ -286,7 +388,7 @@ def test_flask_too_large_raw_request(sentry_init, input_char, capture_events, ap
     response = client.post("/", data=data)
     assert response.status_code == 200
 
-    event, = events
+    (event,) = events
     assert event["_meta"]["request"]["data"] == {
         "": {"len": 2000, "rem": [["!config", "x", 0, 2000]]}
     }
@@ -312,7 +414,7 @@ def test_flask_files_and_form(sentry_init, capture_events, app):
     response = client.post("/", data=data)
     assert response.status_code == 200
 
-    event, = events
+    (event,) = events
     assert event["_meta"]["request"]["data"]["foo"] == {
         "": {"len": 2000, "rem": [["!limit", "x", 509, 512]]}
     }
@@ -370,7 +472,7 @@ def test_logging(sentry_init, capture_events, app):
     client = app.test_client()
     client.get("/")
 
-    event, = events
+    (event,) = events
     assert event["level"] == "error"
 
 
@@ -414,11 +516,11 @@ def test_wsgi_level_error_is_caught(
     with pytest.raises(ZeroDivisionError) as exc:
         client.get("/")
 
-    error, = exceptions
+    (error,) = exceptions
 
     assert error is exc.value
 
-    event, = events
+    (event,) = events
     assert event["exception"]["values"][0]["mechanism"]["type"] == "wsgi"
 
 
@@ -441,7 +543,7 @@ def test_500(sentry_init, capture_events, app):
     client = app.test_client()
     response = client.get("/")
 
-    event, = events
+    (event,) = events
     assert response.data.decode("utf-8") == "Sentry error: %s" % event["event_id"]
 
 
@@ -468,7 +570,7 @@ def test_error_in_errorhandler(sentry_init, capture_events, app):
 
     event1, event2 = events
 
-    exception, = event1["exception"]["values"]
+    (exception,) = event1["exception"]["values"]
     assert exception["type"] == "ValueError"
 
     exception = event2["exception"]["values"][-1]
@@ -560,20 +662,34 @@ def test_errorhandler_for_exception_swallows_exception(
 def test_tracing_success(sentry_init, capture_events, app):
     sentry_init(traces_sample_rate=1.0, integrations=[flask_sentry.FlaskIntegration()])
 
+    @app.before_request
+    def _():
+        set_tag("before_request", "yes")
+
+    @app.route("/message_tx")
+    def hi_tx():
+        set_tag("view", "yes")
+        capture_message("hi")
+        return "ok"
+
     events = capture_events()
 
     with app.test_client() as client:
-        response = client.get("/message")
+        response = client.get("/message_tx")
         assert response.status_code == 200
 
     message_event, transaction_event = events
 
     assert transaction_event["type"] == "transaction"
-    assert transaction_event["transaction"] == "hi"
-    assert "status" not in transaction_event["contexts"]["trace"]
+    assert transaction_event["transaction"] == "hi_tx"
+    assert transaction_event["contexts"]["trace"]["status"] == "ok"
+    assert transaction_event["tags"]["view"] == "yes"
+    assert transaction_event["tags"]["before_request"] == "yes"
 
     assert message_event["message"] == "hi"
-    assert message_event["transaction"] == "hi"
+    assert message_event["transaction"] == "hi_tx"
+    assert message_event["tags"]["view"] == "yes"
+    assert message_event["tags"]["before_request"] == "yes"
 
 
 def test_tracing_error(sentry_init, capture_events, app):
@@ -594,10 +710,10 @@ def test_tracing_error(sentry_init, capture_events, app):
 
     assert transaction_event["type"] == "transaction"
     assert transaction_event["transaction"] == "error"
-    assert transaction_event["contexts"]["trace"]["status"] == "failure"
+    assert transaction_event["contexts"]["trace"]["status"] == "internal_error"
 
     assert error_event["transaction"] == "error"
-    exception, = error_event["exception"]["values"]
+    (exception,) = error_event["exception"]["values"]
     assert exception["type"] == "ZeroDivisionError"
 
 
@@ -617,7 +733,7 @@ def test_class_based_views(sentry_init, app, capture_events):
         response = client.get("/hello-class/")
         assert response.status_code == 200
 
-    event, = events
+    (event,) = events
 
     assert event["message"] == "hi"
     assert event["transaction"] == "hello_class"
