@@ -11,16 +11,15 @@ def test_crumb_capture(sentry_init, capture_events):
     events = capture_events()
 
     response = requests.get("https://httpbin.org/status/418")
-    assert response.status_code == 418
     capture_message("Testing!")
 
-    event, = events
-    crumb, = event["breadcrumbs"]
+    (event,) = events
+    (crumb,) = event["breadcrumbs"]["values"]
     assert crumb["type"] == "http"
     assert crumb["category"] == "httplib"
     assert crumb["data"] == {
         "url": "https://httpbin.org/status/418",
         "method": "GET",
-        "status_code": 418,
-        "reason": "I'M A TEAPOT",
+        "status_code": response.status_code,
+        "reason": response.reason,
     }
